import './ship.dart';
import './board.dart';
import './player.dart';
import './logger.dart';
import 'dart:math';

enum GAMESTATE {
	CREATED,
	INITIALIZED,
	PLAYING,
	OVER
}

class Game {
	List<Player> players = new List<Player>();
	GAMESTATE gamestate = GAMESTATE.CREATED;
	int lastPlayed = 0;
	int size;
	int iterations;

	Game(this.size) {}
	void Init() {
		bool error = true;
		while(error) {
			error = false;

			//Generate set of ships, so both players have same set
			Forces f = new Forces();
			f.Generate(size);

			//Create player
			Player p = new Player(size);
			p.id = 1;
			//Let him arrange ships
			//This may fail, in that case there is while
			//There is bigger chance of failure with bigger size
			error = error || !p.Arrange(f);
			if(error){
				continue;
			}
			players.add(p);

			//Same with second player
			p = new Player(size);
			p.id = 2;
			error = error || !p.Arrange(f);
			if(error){
				continue;
			}
			players.add(p);
			Logger.PrintS(LOGLEVEL.VERBOSE, "Ship list: ${f}");
			Logger.PrintS(LOGLEVEL.DEBUG, "Player 1 created");
			Logger.PrintS(LOGLEVEL.DEBUG, "Player 2 created");
		}
		Logger.PrintS(LOGLEVEL.DEBUG, "Game initialized");
		gamestate = GAMESTATE.INITIALIZED;
	}
	/**
	 * Standard game loop
	 */
	void Start() {
		Init();
		if(gamestate != GAMESTATE.INITIALIZED) {
			Logger.PrintS(LOGLEVEL.MINIMAL, "Game needs to be initialized first.");
			Logger.PrintS(LOGLEVEL.MINIMAL, "Please initialize it and then try starting again.");
			return;
		}
		Logger.PrintS(LOGLEVEL.MINIMAL, "Game started with size ${size}.");
		gamestate = GAMESTATE.PLAYING;
		//Randomly select starting player
		int lastplayed = new Random().nextInt(2);
		iterations = 0;
		while(gamestate != GAMESTATE.OVER) {
			Position position = new List<int>(2);
			int activePlayer, nonActivePlayer;
			if(lastplayed == 0) {
				activePlayer = 1;
				nonActivePlayer = 0;
			} else {
				activePlayer = 0;
				nonActivePlayer = 1;
			}

			//Now it's pirates turn
			//FIRE!!
			position = players[activePlayer].Shot();
			//Ask God for shoot results
			bool isHit = players[nonActivePlayer].IsHit(position);
			//Tell the crew they suck
			players[activePlayer].WasHit(position, isHit);

			//switch last player
			lastplayed = activePlayer;

			++iterations;
			//Every even half-turn report status
			if(iterations % 2 == 0) {
				Logger.PrintS(LOGLEVEL.DEBUG, "P1 ship status:");
				players[0].PrintShipStatus(LOGLEVEL.DEBUG);
				Logger.PrintS(LOGLEVEL.DEBUG, "P2 ship status:");
				players[1].PrintShipStatus(LOGLEVEL.DEBUG);
			}
			_CheckGameState();
		}
	}

	void _CheckGameState() {
		if(players[0].AllShipsSink()) {
			Logger.PrintS(LOGLEVEL.MINIMAL, "Game over.");
			Logger.PrintS(LOGLEVEL.MINIMAL, "P1 wins.");
			Logger.PrintS(LOGLEVEL.NORMAL, "Game took ${(iterations/2).round()} turns.");			
			gamestate = GAMESTATE.OVER;
		} else if (players[1].AllShipsSink()) {
			Logger.PrintS(LOGLEVEL.MINIMAL, "Game over.");
			Logger.PrintS(LOGLEVEL.MINIMAL, "P2 wins.");
			Logger.PrintS(LOGLEVEL.NORMAL, "Game took ${(iterations/2).round()} turns.");			
			gamestate = GAMESTATE.OVER;
		}
	}

}