import 'dart:io';


enum LOGLEVEL {
	MINIMAL,
	NORMAL,
	VERBOSE,
	DEBUG,
}

/**
 * Class for console and logfile output
 * Static singleton class
 */
class Logger {
	static final Logger _logger = new Logger._internal();
	LOGLEVEL _level;

	bool logToFile = true;
	String logfile = "../game.log";
	String prefix = "";
	//Prefix example, can contain code
	//String prefix = "${new DateTime.now().hashCode}";

	factory Logger() {
		return _logger;
	}
	Logger._internal();

	static void SetLogLevelS(LOGLEVEL level) {
		new Logger().SetLogLevel(level);
	}
	void SetLogLevel(LOGLEVEL level) {
		this._level = level;
	}

	static void PrintS(LOGLEVEL msgLevel, String message) {
		new Logger().Print(msgLevel, message);
	}
	void Print(LOGLEVEL msgLevel, String message) {
		String str;
		//Send it to console
		if(msgLevel.index <= _level.index) {
			String lvl = Logger.LoglevelToString(msgLevel);
			str = (prefix!=""?"[${prefix}]> ${message}":"${message}");
			print(str);
		}
		//Send it to file, asynchronously
		if(logToFile) {
			str = (prefix!=""?"[${LoglevelToString(msgLevel)}][${prefix}]> ${message}\r\n":"[${LoglevelToString(msgLevel)}]>${message}\r\n");
			new File(logfile).writeAsString(str, mode: FileMode.APPEND);
		}
	}

	/**
	 * Translate enum to string
	 * Default conversion contains name of enum
	 */
	static String LoglevelToString(LOGLEVEL level) {
		switch(level) {
			case LOGLEVEL.MINIMAL:
				return "Minimal";
			case LOGLEVEL.NORMAL:
				return "Normal";
			case LOGLEVEL.VERBOSE:
				return "Verbose";
			case LOGLEVEL.DEBUG:
				return "Debug";
		}
	}

	/**
	 * Translate int to enum
	 */
	static LOGLEVEL IntToLoglevel(int level) {
		switch(level) {
			case 0:
				return LOGLEVEL.MINIMAL;
			case 1:
				return LOGLEVEL.NORMAL;
			case 2:
				return LOGLEVEL.VERBOSE;
			case 3:
				return LOGLEVEL.DEBUG;
			default:
				return null;
		}
	}
	
}
