import './board.dart';
import './ship.dart';
import './logger.dart';
import 'dart:math';


class Player {
	PlayerBoard pBoard;
	int id;

	Player(int size){
		pBoard = new PlayerBoard(size);
	}

	/**
	 * Randomly place ships on battlefield
	 */
	bool Arrange(Forces forces) {
		int maxTries = 100;
		bool error = false;
		int i = 0;
		Random rng = new Random();
		while(true) {
			for(var ship in forces.ships) {
				//Randomly rotate
				if(rng.nextInt(2) == 1) {
					ship = new Ship(0, 0, ship.dimensions.height, ship.dimensions.width);
				}
				//Gets available location
				var pos = pBoard.GetFreePosition(ship.dimensions.width, ship.dimensions.height);
				if(pos == null) {
					error = true;
					break;
				}
				//Place ship
				bool success = pBoard.AddShip(new Ship(pos.x, pos.y, ship.dimensions.width, ship.dimensions.height));
				//Should't fail, but u kno... just to be sure
				if(!success) {
					error = true;
					break;
				}
			}
			if(++i > maxTries) {
				ResetBoard();
				break;
			}
			if(error) {
				error = false;
				ResetBoard();
			} else {
				break;
			}
		}
		return !error;
	}

	/**
	 * Helper method, just to reset board
	 */
	void ResetBoard(){
		pBoard = new PlayerBoard(pBoard.size);
	}

	/**
	 * Finds "best" place to shot
	 */
	Position Shot() {
		int x, y;
		SEASTAT st = SEASTAT.HIT;
		List<List<SEASTAT>> possiblePositions = pBoard.possiblePositions;
		Random rng = new Random();
		int iter = 0;
		int max = rng.nextInt(25)+50;
		while(st != SEASTAT.NOTDISC) {
			if(iter == max) {
				//In advanced game stage could rapidly slow down finding
				//This takes first UNDISCOVERED field
				for(int i = 0; i < pBoard.size; ++i) {
					for(int j = 0; j < pBoard.size; ++j) {
						if(possiblePositions[i][j] == SEASTAT.NOTDISC) {
							return new Position(i, j);
						}
					}
				}
			}
			//Rodman
			x = rng.nextInt(pBoard.size);
			//Ardonm
			y = rng.nextInt(pBoard.size);
			st = possiblePositions[x][y];
			++iter;
		}
		return new Position(x, y);
	}

	/**
	 * true - hit
	 * false - u missed, lul
	 */
	bool IsHit(Position position) {
		Rectangle r = new Rectangle(position.x, position.y, 1, 1);
		bool result = false;
		for(Ship ship in pBoard.GetShips()) {
			//Go ship by ship and check for collision
			bool tmp = ship.dimensions.containsRectangle(r);
			if(tmp) {
				ship.Hit();
			}
			result = result || tmp;
			if(result) {
				break;
			}
		}
		if(result) {
			Logger.PrintS(LOGLEVEL.VERBOSE, "P${id}:${position.x}x${position.y}-HIT");
		} else {
			Logger.PrintS(LOGLEVEL.DEBUG, "P${id}:${position.x}x${position.y}-Nothing");
		}
		return result;
	}

	/**
	 * Reports to crew if they missed or not and save it to map
	 */
	void WasHit(Position position, bool isHit) {
		if(isHit) {
			pBoard.possiblePositions[position.x][position.y] = SEASTAT.HIT;
		} else {
			pBoard.possiblePositions[position.x][position.y] = SEASTAT.WATER;
		}
	}

	/**
	 * Helper function, finds out if there is something more than just wreckage
	 */
	bool AllShipsSink() {
		bool result = true;
		for(Ship ship in pBoard.GetShips()) {
			result = result && ship.IsSink();
		}
		return result;
	}

	/**
	 * Prints all ships
	 */
	void PrintShipStatus([LOGLEVEL level]) {
		level = level ?? LOGLEVEL.VERBOSE;
		for(Ship ship in pBoard.GetShips()) {
			Logger.PrintS(level, "${ship}:${ship.GetHealth()}/${ship.GetMaxHealth()}");
		}
	}

}
